// Includes I didn't make myself
#include <iostream>
#include <cmath>

// Includes I did make myself
#include "functions.hpp"
#include "player.hpp"
#include "target.hpp"

Target::Target(sf::Texture& textureTarget, sf::Texture& textureArrow) : position(rand() % MAP_W, rand() % MAP_H) {
	sprite.setTexture(textureTarget);
	
	spriteArrow.setTexture(textureArrow);
	spriteArrow.setOrigin(20, 35);
	spriteArrow.setPosition(SCR_W / 2, SCR_H / 2);
	spriteArrow.setColor(sf::Color(255, 255, 255, 100));
	
	to.x = rand() % MAP_W;
	to.y = rand() % MAP_H;
	currentAng = atan2(to.y - position.y, to.x - position.x) + 2 * M_PI; // HERE
	sprite.setRotation(radToDeg(currentAng));
	sprite.setOrigin(75, 20);
	
	
	flipTimer.restart();
	flip = -1;
}

Target::~Target() {

}

int Target::logic(Player& player, bool hintArrow) {
	sf::Vector2f dispPos = player.toScreen(position);
	sprite.setPosition(dispPos);
	
	sf::FloatRect spriteBounds = sprite.getGlobalBounds();
	sf::FloatRect boundingBox(position.x - spriteBounds.width / 2, position.y - spriteBounds.height / 2, spriteBounds.width, spriteBounds.height);
	
	if (player.isBiting()) {
		if (player.getBiteRect().intersects(boundingBox)) {
			player.biteSuccess();
			return TRGT_DESTROY;
		}
	}

	if (pythagoras(position, to) < 200) {
		to.x = rand() % MAP_W;
		to.y = rand() % MAP_H;
	}
	
	float targetAng = atan2(to.y - position.y, to.x - position.x) + 2 * M_PI;
	
	float alpha = targetAng - currentAng;
	float beta = targetAng - currentAng + M_PI * 2;
	float theta = targetAng - currentAng - M_PI * 2;
	
	float fa = fabs(alpha);
	float fb = fabs(beta);
	float fc = fabs(theta);
	float check;
	
	if (fa < fb && fa < fc) {
		check = alpha;
	} else if (fb < fc) {
		check = beta;
	} else {
		check = theta;
	}
	
	if (check > 0) { // CLOCKWISE
		currentAng += fmin(fabs(check), TRGT_ROTSPEED);
	} else { // COUNTERCLOCKWISE
		currentAng -= fmin(fabs(check), TRGT_ROTSPEED);
	}
	
	if (currentAng > M_PI * 2) currentAng -= M_PI * 2;
	if (currentAng < 0) currentAng += M_PI * 2;
	
	position.x += cos(currentAng) * TRGT_MOVSPEED;
	position.y += sin(currentAng) * TRGT_MOVSPEED;
	sprite.setRotation(radToDeg(currentAng));
	
	if (flipTimer.getElapsedTime().asMilliseconds() > 1200) {
		sprite.setScale(1, flip);
		flipTimer.restart();
		flip *= -1;
	}
	
	drawHintArrow = hintArrow;
	if (hintArrow) {
		float arrowAng;
		sf::Vector2f playerPos = player.getPosition();
		arrowAng = atan2(position.y - playerPos.y, position.x - playerPos.x);
		spriteArrow.setRotation(radToDeg(arrowAng));
	}
	
	return TRGT_DONE;
}

int Target::render(sf::RenderWindow& window) {
	// std::cout << "drawn?";
	// if (dstBox.width != 0) {
		// drawRect(window, sf::IntRect(dstBox), sf::Color::Green);
		// std::cout << "drawn";
	// }
	
	window.draw(sprite);
	
	if (drawHintArrow) window.draw(spriteArrow);
	
	return 0;
}

int Target::bite(sf::FloatRect area) {
	return 0;
}
